package com.miaxis.wx.service.impl;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.miaxis.common.constant.Constants;
import com.miaxis.common.core.redis.RedisCache;
import com.miaxis.common.exception.CustomException;
import com.miaxis.common.sms.MD5Utils;
import com.miaxis.common.utils.StringUtils;
import com.miaxis.feign.dto.FilmDTO;
import com.miaxis.feign.dto.FilmMcpData;
import com.miaxis.feign.dto.FilmXdResult;
import com.miaxis.feign.dto.fulu.FuluCommonApiDTO;
import com.miaxis.feign.dto.fulu.FuluDTO;
import com.miaxis.feign.dto.fulu.FuluGoodsInfo;
import com.miaxis.feign.dto.fulu.FuluResult;
import com.miaxis.feign.service.IFuluService;
import com.miaxis.feign.service.IWxgetTokenService;
import com.miaxis.system.service.ISysConfigService;
import com.miaxis.wx.domain.WxOrder;
import com.miaxis.wx.dto.*;
import com.miaxis.wx.mapper.WxOrderMapper;
import com.miaxis.wx.service.IRefundRecordService;
import com.miaxis.wx.service.IWxOrderService;
import com.miaxis.wx.service.IWxXcxMessageService;
import lombok.extern.slf4j.Slf4j;
import org.apache.http.HttpResponse;
import org.apache.http.HttpStatus;
import org.apache.http.client.entity.UrlEncodedFormEntity;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClientBuilder;
import org.apache.http.message.BasicNameValuePair;
import org.apache.http.util.EntityUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.scheduling.concurrent.ThreadPoolTaskExecutor;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

import static com.miaxis.common.utils.OrderCodeFactory.getOrderCode;

/**
 * 电影订单Service业务层处理
 *
 * @author miaxis
 * @date 2021-05-07
 */
@Service
@Slf4j
public class WxXcxMesageServiceImpl implements IWxXcxMessageService {


    @Resource
    private RedisCache redisCache;

    @Autowired
    private IWxgetTokenService wxgetTokenService;

    @Value("${app.appid}")
    private String appid;

    @Value("${app.secret}")
    private String secret;


    @Override
    public String getXcxMessageToken() {
        Object cacheObject = redisCache.getCacheObject(Constants.XCX_MESSAGE_TOKEN);
        //如果过期，重新获取并保存到redis
        if (cacheObject == null){
            String result = wxgetTokenService.getAccessToken("client_credential",appid,secret);
            JSONObject json = JSONObject.parseObject(result);
            String token = json.getString("access_token");
            int expiresIn = json.getIntValue("expires_in");
            redisCache.setCacheObject(Constants.XCX_MESSAGE_TOKEN,token,expiresIn, TimeUnit.SECONDS);
            return token;
        }else {
            return String.valueOf(cacheObject);
        }

    }
}
